import asyncio
from datetime import datetime, timedelta
from telegram import Bot, Update
from telegram.ext import Application, CommandHandler, MessageHandler, filters, ContextTypes
import aiohttp

# BOT_TOKEN = "YOUR_BOT_TOKEN_HERE"  # Replace with your actual token
# Loop Gatekeeper bot
BOT_TOKEN = "7664853316:AAEP2dAb3GuWLib6eJB1vu8DIR65HqlIBRA"
# Loop Development
# BOT_TOKEN = "7309367551:AAGJNNPPQroL7yqw7U7eeUmsOXrKVQgzAw0" 
KEYWORD_LIST_URL = "https://loopaddon.uk/files/PJ/BOT_DATA/loop_bot_keywords.json"

new_users = {}
application = None  # Global application instance
user_warnings = {}

check_list1 = []
check_list2 = []
check_list3 = []

async def is_user_admin(chat, user_id):
    administrators = await chat.get_administrators()
    return any(admin.user.id == user_id or admin.is_anonymous for admin in administrators)

def log_debug(message):
    print(f"[DEBUG] {message}")
    with open("/srv/http/jet/LOOP_TEST/laugh/bot.log", "a") as f:
        f.write(f"[DEBUG] {message}\n")

async def fetch_keyword_lists(context=None):
    global check_list1, check_list2, check_list3
    try:
        log_debug("Fetching keyword lists...")
        async with aiohttp.ClientSession() as session:
            async with session.get(KEYWORD_LIST_URL) as response:
                if response.status == 200:
                    data = await response.json()
                    check_list1 = data.get("check_list1", [])
                    check_list2 = data.get("check_list2", [])
                    check_list3 = data.get("check_list3", [])
                    log_debug(f"Keywords updated: {len(check_list1)}, {len(check_list2)}, {len(check_list3)} items")
                else:
                    log_debug(f"Failed to fetch keyword lists. HTTP Status: {response.status}")
    except Exception as e:
        log_debug(f"Error fetching keyword lists: {str(e)}")
    
    return None

async def start(update, context):
    user_id = update.effective_user.id
    chat = update.effective_chat

    if not await is_user_admin(chat, user_id):
        await update.message.reply_text("You need to be an admin to use this command.")
        return

    await update.message.reply_text(
        "Gate Keeper is active! \n\n"
        "Be aware there is a 3 strike warning process in place. \n\n"
        "Any user that receives 3 warnings will be automatically removed from the group."
    )

async def welcome_new_user(update, context):
    if update.message.new_chat_members:
        for user in update.message.new_chat_members:
            user_id = user.id
            chat_id = update.message.chat.id
            first_name = user.first_name

            try:
                user_photos = await context.bot.get_user_profile_photos(user_id)
                pic_count = user_photos.total_count
            except:
                pic_count = 0

            if pic_count == 0:
                await context.bot.send_photo(
                    chat_id=chat_id,
                    photo='https://i.imgur.com/q3l7yOM.jpeg',
                    caption=(
                        f"Welcome {first_name}!\n\n"
                        "Please ensure you have a profile picture as it's a requirement of this group. "
                        "You will be automatically removed from this group if you do not have a profile picture within 24 hours of joining."
                    )
                )
            else:
                await context.bot.send_message(
                    chat_id=chat_id,
                    text=(
                        f"Welcome {first_name}!\n\n"
                        "Please see pinned post for installation instructions."
                    )
                )

            new_users[user_id] = {"chat_id": chat_id, "join_time": datetime.now()}

async def warn_user(update, context):
    user_id = update.effective_user.id
    chat = update.effective_chat

    if not await is_user_admin(chat, user_id):
        await update.message.reply_text("You need to be an admin to use this command.")
        return

    if update.message.reply_to_message:
        warned_user = update.message.reply_to_message.from_user
        target_id = warned_user.id
        chat_id = update.message.chat.id

        user_warnings[target_id] = user_warnings.get(target_id, 0) + 1
        warnings = user_warnings[target_id]

        await update.message.reply_text(
            f"{warned_user.first_name} has been warned. \nTotal warnings: {warnings}."
        )

        if warnings >= 3:
            try:
                await context.bot.ban_chat_member(chat_id, target_id)
                await context.bot.send_message(
                    chat_id, f"{warned_user.first_name} has been removed for receiving 3 warnings."
                )
                del user_warnings[target_id]
            except Exception as e:
                await update.message.reply_text(f"Failed to remove {warned_user.first_name}. Error: {e}")
    else:
        await update.message.reply_text("Please reply to a user's message to warn them.")

async def reset_warnings(update, context):
    user_id = update.effective_user.id
    chat = update.effective_chat

    if not await is_user_admin(chat, user_id):
        await update.message.reply_text("You need to be an admin to use this command.")
        return

    if update.message.reply_to_message:
        warned_user = update.message.reply_to_message.from_user
        target_id = warned_user.id

        if target_id in user_warnings:
            del user_warnings[target_id]
            await update.message.reply_text(f"Warnings for {warned_user.first_name} have been reset.")
        else:
            await update.message.reply_text(f"{warned_user.first_name} has no warnings.")
    else:
        await update.message.reply_text("Please reply to a user's message to reset their warnings.")

async def ban_user(update, context):
    user_id = update.effective_user.id
    chat = update.effective_chat

    if not await is_user_admin(chat, user_id):
        await update.message.reply_text("You need to be an admin to use this command.")
        return

    if update.message.reply_to_message:
        banned_user = update.message.reply_to_message.from_user
        target_id = banned_user.id
        chat_id = update.message.chat.id

        reason = " ".join(context.args) if context.args else "No reason provided."

        try:
            await context.bot.ban_chat_member(chat_id, target_id)
            await context.bot.send_message(
                chat_id, f"{banned_user.first_name} has been banned. \nReason: {reason}"
            )
        except Exception as e:
            await update.message.reply_text(f"Failed to ban {banned_user.first_name}. Error: {e}")
    else:
        await update.message.reply_text("Please reply to a user's message to ban them.")

async def handle_message(update, context):
    if not update.message or not update.message.text:
        return

    text = update.message.text.lower()
    log_debug(f"Received message: {text}")
    user_id = update.message.from_user.id
    first_name = update.message.from_user.first_name
    chat_id = update.message.chat.id

    async def issue_warning():
        if user_id not in user_warnings:
            user_warnings[user_id] = 0
        user_warnings[user_id] += 1
        warnings = user_warnings[user_id]

        await update.message.reply_text(
            f"Warning {first_name}! \nThis is warning {warnings}."
        )

        if warnings >= 3:
            try:
                await context.bot.ban_chat_member(chat_id, user_id)
                await context.bot.send_message(
                    chat_id, f"{first_name} has been removed for receiving 3 warnings."
                )
                del user_warnings[user_id]
            except Exception as e:
                await update.message.reply_text(f"Failed to remove {first_name}. Error: {e}")

    if any(check in text for check in check_list1):
        log_debug(f"Matched check_list1 for message: {text}")
        await update.message.reply_text(
            f"Warning {first_name}! \n\n"
            "The answer is always the same:\n\n"
            "If you've looked, you know the answer.\n\n"
            "If you've not looked, it's not there.\n\n"
            "If-Its-On-Its-In"
        )
        await issue_warning()
    elif any(check in text for check in check_list2):
        log_debug(f"Matched check_list2 for message: {text}")
        await update.message.reply_text(
            f"Warning {first_name}! \n\n"
            "Replays are added and announced as they become available."
        )
        await issue_warning()
    elif any(check in text for check in check_list3):
        log_debug(f"Matched check_list3 for message: {text}")
        await update.message.reply_text(
            f"{first_name} \n"
            "Please see pinned post for installation instructions."
        )
    else:
        log_debug("No keyword matches found")

async def remove_users_without_profile_pics(context: ContextTypes.DEFAULT_TYPE = None):
    now = datetime.now()
    to_remove = []
    
    # Get bot instance from application if context is None
    bot = context.bot if context else application.bot

    for user_id, data in new_users.items():
        join_time = data['join_time']
        chat_id = data['chat_id']

        if (now - join_time).total_seconds() >= 86400:
            try:
                user_photos = await bot.get_user_profile_photos(user_id)
                if user_photos.total_count == 0:
                    await bot.ban_chat_member(chat_id, user_id)
                    await bot.unban_chat_member(chat_id, user_id)
                    await bot.send_message(
                        chat_id, f"User ID {user_id} has been removed for not setting a profile picture."
                    )
                to_remove.append(user_id)
            except Exception as e:
                print(f"Error checking/removing user {user_id}: {e}")

    for uid in to_remove:
        del new_users[uid]

async def main():
    global application
    try:
        log_debug("Starting bot...")
        # Build application
        application = Application.builder().token(BOT_TOKEN).build()
        
        # Add handlers
        application.add_handler(CommandHandler("start", start))
        application.add_handler(CommandHandler("warn", warn_user))
        application.add_handler(CommandHandler("reset", reset_warnings))
        application.add_handler(CommandHandler("ban", ban_user))
        application.add_handler(MessageHandler(filters.StatusUpdate.NEW_CHAT_MEMBERS, welcome_new_user))
        application.add_handler(MessageHandler(filters.TEXT & ~filters.COMMAND, handle_message))
        log_debug("Handlers added")

        # First fetch of keywords
        await fetch_keyword_lists()
        
        # Set up periodic jobs
        job_queue = application.job_queue
        job_queue.run_repeating(fetch_keyword_lists, interval=1800, first=1)
        job_queue.run_repeating(remove_users_without_profile_pics, interval=1800, first=1)
        log_debug("Jobs scheduled")
        
        print("Bot is running...")
        log_debug("Starting polling...")
        await application.run_polling(drop_pending_updates=True, close_loop=False)
    except Exception as e:
        log_debug(f"Bot error: {str(e)}")
        raise

if __name__ == "__main__":
    try:
        asyncio.run(main())
    except KeyboardInterrupt:
        print("Bot stopped by user")
    except Exception as e:
        print(f"Fatal error: {e}")